library(plyr)
library(ggplot2)
library(sqldf)
require(grid)

args <- commandArgs(trailingOnly = TRUE)
inputFile=args[1]
outputFile=args[2]
title=args[3]

if ( length(args) != 3) {
  stop("three arguments needed")
} else {

# read the runtime data
DF <- read.table(inputFile,header=TRUE)

# compute the mean, standard deviation and standard error (though only the mean is used
# in this PLDI, to calculate speedup.
cdata <- ddply(DF, .(Variant,Cores), summarise, N = length(Runtime), mean=round(mean(Runtime),2), sd=round(sd(Runtime),2), se=round(sd(Runtime)/sqrt(length(Runtime)),2))

# compute speedup vs runtime using 1 core of the same variant (i.e. the same skeleton)
withOneNode <- function(variant){
  x <- sqldf(paste("select mean from cdata where Cores == 1 and Variant = '", variant, "'", sep=""))
  return(x)
}

# calculate speedup from runtimes
calcSpeedUp <- function(row){
    thisVariant <- row$Variant
    speedOnOneNode <- withOneNode(thisVariant)
    speedup <- round(speedOnOneNode / row$mean,2)
    row$speedup <- as.numeric(speedup)
    return (row)
}

# removes gridlines
none <- element_blank()

pdf(paste(outputFile,"-runtime.pdf",sep=""))

# show runtime
bp <- ggplot(cdata, aes(x=Cores, y=mean, group=Variant)) + geom_errorbar(aes(ymin=mean-se, ymax=mean+se), width=.1) + geom_line(aes(linetype=Variant),size=1.2) + geom_point(aes(shape=Variant),size=4) + ylab("Runtime (Seconds)")  + labs(title = title)

print(bp + theme_bw() + theme(legend.key = element_blank(), legend.justification=c(1,1), legend.position=c(1,1), legend.key.width=unit(3,"line"), legend.title=element_blank(), text = element_text(size=18), panel.grid.major = none, panel.grid.minor = none, panel.background = element_rect(fill='white', colour='black')) + scale_x_continuous(expand = c(0, 0)) + scale_y_continuous(expand = c(0, 0)))

}

dev.off()
