library(plyr)
library(ggplot2)
library(sqldf)
require(grid)

args <- commandArgs(trailingOnly = TRUE)
inputFile=args[1]
outputFile=args[2]
title=args[3]
failureFreeLazy=args[4]
failureFreeLazySkel=args[5]
failureFreeEager=args[6]
failureFreeEagerSkel=args[7]

#if ( length(args) != 4) {
#  stop("four arguments needed")
#} else {

DF <- read.table(inputFile,header=TRUE)

cdata <- ddply(DF, .(Variant,TimeOfFail), summarise, N = length(Runtime), mean=round(mean(Runtime),2), sd=round(sd(Runtime),2), se=round(sd(Runtime)/sqrt(length(Runtime)),2))

# helper
printf <- function(...)print(sprintf(...))

# withOneNode <- function(variant){
#   x <- sqldf(paste("select mean from cdata where TimeOfFail == 1 and Variant = '", variant, "'", sep=""))
#   return(x)
# }

# calcSpeedUp <- function(row){
#     thisVariant <- row$Variant
#     speedOnOneNode <- withOneNode(thisVariant)
#     speedup <- round(speedOnOneNode / row$mean,2)
#     row$speedup <- as.numeric(speedup)
#     return (row)
# }

# removes gridlines
none <- element_blank()

pdf(paste(outputFile,"-runtime.pdf",sep=""))

# show runtime
bp <- ggplot(cdata, aes(x=TimeOfFail, y=mean, group=Variant)) + geom_errorbar(aes(ymin=mean-se, ymax=mean+se), width=.1) + geom_line(aes(linetype=Variant),size=2) + geom_point(aes(shape=Variant)) + ylab("Runtime (Seconds)") + xlab("Time of Simultanous 5 Node Failure (Seconds)") + labs(title = title) + geom_hline(yintercept = as.numeric(failureFreeLazy), linetype="dashed") + annotate("text", min(22), as.numeric(failureFreeLazy)+3, label = failureFreeLazySkel,size=5) + geom_hline(yintercept = as.numeric(failureFreeEager), linetype="dashed") + annotate("text", min(22), as.numeric(failureFreeEager)+3, label = failureFreeEagerSkel,size=5)

print(bp + theme_bw()  + theme(legend.key = element_blank(), legend.justification=c(1,1), legend.position=c(0.7,1), legend.key.width=unit(3,"line"), text= element_text(size=18), panel.grid.major = none, panel.grid.minor = none, panel.background = element_rect(fill='white', colour='black')))

write.table(cdata,file=paste(outputFile,"-timeoffailure.csv",sep=""),sep=",",row.names=F)

# }

dev.off()
